/*
 * Created on 19.05.2005 for PIROL
 * 
 * SVN header information:
 * $Author: LBST-PF-3\orahn $
 * $Rev: 2446 $
 * $Date: 2006-09-12 14:57:25 +0200 (Di, 12 Sep 2006) $
 * $Id: WizardHeaderPanel.java 2446 2006-09-12 12:57:25Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.ui.wizard;

import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Font;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

/**
 * This is the <code>Panel</code> shown at the top of the 
 * <code>{@link WizardDialog}</code>. It shows a decribing text at the top left
 * corner and an image at the top right corner just behind the text.
 * @author Carsten Schulze
 * @author FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * Project: PIROL (2005),
 * Subproject: Daten- und Wissensmanagement 
 */
public class WizardHeaderPanel extends JPanel {
    private static final ImageIcon DEFAULT_ICON = 
        new ImageIcon("de.fhOsnabrueck.jump.pirol/dialogs/wizard/wizards_hat_and_wand.png");
    
    private static final String DEFAULT_STRING = new String("Wizard");
    /**
     * For debugging and testing this class.
     * @param args ignored
     */
    public static void main(String[] args){
        WizardHeaderPanel hp = new WizardHeaderPanel("Beschreibung",null);
        
        JFrame frame = new JFrame();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.getContentPane().add(hp);
        frame.setSize(hp.getSize());
        frame.setVisible(true);      
    }
    private JLabel imageLabel;
    private JTextArea textArea;
    /**
     * The Panel will have the size of 400 x 75 Pixel.
     * @param description the description text to be printed in the top left 
     * corner of this panel.
     * @param image the image that will be setteled in the top right corner of 
     * this panel.
     */
    public WizardHeaderPanel(String description, ImageIcon image){
        if(description == null){
            description = DEFAULT_STRING;
        }
        if(image == null){
            image = DEFAULT_ICON;
        }
        
        setLayout(null);
        setBackground(null);
        setSize(400,75);
        setPreferredSize(getSize());
        
        imageLabel = new JLabel(image);
        imageLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        imageLabel.setAlignmentY(Component.TOP_ALIGNMENT);
        
        JPanel imagePanel = new JPanel();
        imagePanel.setOpaque(false);
        imagePanel.setSize(this.getSize());
        imagePanel.setLocation(0,0);
        imagePanel.setLayout(new FlowLayout(FlowLayout.RIGHT,0,0));
        imagePanel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        imagePanel.setAlignmentY(Component.TOP_ALIGNMENT);
        imagePanel.add(imageLabel);
        
        textArea = new JTextArea(description);
        textArea.setOpaque(false);
        textArea.setFont(new Font("SansSerif",Font.BOLD,14));
        textArea.setSize(this.getSize().width-10,this.getSize().height-10);
        textArea.setLocation(5,5);
        textArea.setFocusable(false);
        
        add(textArea);
        add(imagePanel);
    }
    
    /**
     * Specifies the background color of this panel.
     * @param color the new background color. If this parameter is 
     * <code>null, Color.WHITE</code> is used. 
     */
    public void setBackground(Color color) {
        if(color == null){
            super.setBackground(Color.WHITE);
        }
        else{
            super.setBackground(color);
        }
    }
    /**
     * This method can be used to change the description text.
     * @param text the new text.
     */
    public void setDescriptionText(String text){
        if(text == null){
            text = DEFAULT_STRING;
        }
        textArea.setText(text);
    }
    /**
     * This method can be used to change the image.
     * @param image the image used as icon.
     */
    public void setImage(ImageIcon image) {
        if(image == null){
            image = DEFAULT_ICON;
            
        }
        imageLabel.setIcon(image);
    }
}
