/*
 * Created on 12.01.2005
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2519 $
 *  $Date: 2006-10-11 11:53:07 +0200 (Mi, 11 Okt 2006) $
 *  $Id: DreiecksInterpolierer.java 2519 2006-10-11 09:53:07Z LBST-PF-3\orahn $s
 */
package de.fhOsnabrueck.jump.pirol.utilities.graph;

import java.util.ArrayList;
import java.util.List;

import com.vividsolutions.jump.feature.Feature;
import com.vividsolutions.jump.feature.FeatureCollection;

import de.fhOsnabrueck.jump.pirol.utilities.comparisonAndSorting.ObjectComparator;
import de.fhOsnabrueck.jump.pirol.utilities.graph.Delaunay.DelaunayDiagrammTools;
import de.fhOsnabrueck.jump.pirol.utilities.graph.Delaunay.DelaunayPunkt;

/**
 * Class to calculate iso lines
 * 
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2005),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2519 $
 */
public class DreiecksInterpolierer extends DelaunayDiagrammTools{
    /**
     * value to construct isolines for
     */
    protected double isoLinienWert = Double.NaN;
    
    /**
     * List to store isolines in
     */
    protected List isoLinien = null;
    
    
    public DreiecksInterpolierer(FeatureCollection punktefeld, DelaunayPunkt[] punktArray, int isoLinienAttribut, double isoLinienWert) {
        super(punktefeld, punktArray, isoLinienAttribut);
        this.isoLinienWert = isoLinienWert;
        this.isoLinien = new ArrayList();

    }
    
    public DreiecksInterpolierer(FeatureCollection punktefeld, DelaunayPunkt[] punktArray, int isoLinienAttribut ) {
        super(punktefeld, punktArray, isoLinienAttribut);
        this.isoLinien = new ArrayList();
    }
    
    public double getIsoLiniesValue() {
        return isoLinienWert;
    }
    public void setIsoLiniesValue(double isoLinienWert) {
        this.isoLinienWert = isoLinienWert;
    }
    
    public List getIsoLinies( double x, double y ) throws Exception{
        if (this.isoLinien.isEmpty()){
            PirolPoint ausgangsPunkt = new PirolPoint( new double[]{x,y}, -1 );
            this.isoLinienWert = this.calculateValueAtPoint(ausgangsPunkt, this.pointsArray);
            return this.getIsoLinies();
        }
        return this.isoLinien;
    }
    
    public List getIsoLinies( double wert ){
        if (this.isoLinien.isEmpty()){
            this.setIsoLiniesValue(wert);
            return this.getIsoLinies();
        }
        return this.isoLinien;
    }
    
    protected List getIsoLinies(){
        PirolEdge k;
        DelaunayPunkt anfangsPunkt,endPunkt,pkt;
        double wertAnfang, wertEnde;
        Feature fAnfang, fEnde;
        
        PirolEdge[] kanten = null;
        
        for (int i = 0; i < this.pointsArray.length; i++) {
            pkt = this.pointsArray[i];
            kanten = pkt.getBildetKantenArray();
            
            for (int j = 0; j < pkt.getBildetKantenArray().length; j++) {
                k = kanten[j];
                if (!k.isGueltig())
                    continue;

                anfangsPunkt = (DelaunayPunkt)k.getAnfang();
                endPunkt = (DelaunayPunkt)k.getEnde();

                fAnfang = this.getFeature(anfangsPunkt.getIndex());
                fEnde = this.getFeature(endPunkt.getIndex());

                wertAnfang = ObjectComparator.getDoubleValue(fAnfang.getAttribute(this.indexOfAttributeOfInterest));
                wertEnde = ObjectComparator.getDoubleValue(fEnde.getAttribute(this.indexOfAttributeOfInterest));

                if (Double.isNaN(wertAnfang) || Double.isNaN(wertEnde)) {
                    logger.printDebug(this.getClass().getName()
                            + ".getIsoLinien: found NAN!");
                }

                if (wertAnfang >= this.isoLinienWert
                        && wertEnde <= this.isoLinienWert) {
                    this.findLinesBeginningAtLine(k, this.isoLinien, this.isoLinienWert);
                } else if (wertAnfang <= this.isoLinienWert
                        && wertEnde >= this.isoLinienWert) {
                    this.findLinesBeginningAtLine(k, this.isoLinien, this.isoLinienWert);
                }
                k.setGueltig(false);
            }
        }
        
        
        for ( int i=0; i<this.pointsArray.length; i++ ){
            pkt = this.pointsArray[i];
            kanten = pkt.getBildetKantenArray();
            for (int j=0; j<pkt.getBildetKantenArray().length; j++)
                kanten[j].setGueltig(true);
        }
  
        return this.isoLinien;        
    }
    
}
